from typing import Dict
import xml.etree.ElementTree as et
import uuid
from abc import ABC, abstractmethod


class Question(ABC):
    """"Abstract Base Class for all questions."""
    
    def __init__(self, identify_id: str, text: str):
        self.identify_id: str = identify_id
        self.text: str = text
        self.internal_id: str = str(uuid.uuid4())
        self.xml: et.Element = self.instantiate_xml()

    @abstractmethod
    def instantiate_xml(self) -> et.Element:
        pass

    def get_xml(self) -> et.Element:
        """Returns the xml structure of the question."""
        return self.xml

    def pretty_print(self) -> str:
        """Prints the structure of the XML with indentation."""
        et.indent(self.xml)
        return et.tostring(self.xml).decode()
    

class CommentQuestion(Question):
    """Class to represent a comment question in a questionnaire."""

    def __init__(self, identify_id: str, text: str):
        self.identify_id: str = identify_id
        self.text: str = text
        self.internal_id: str = str(uuid.uuid4())
        self.xml: et.Element = self.instantiate_xml()

    def instantiate_xml(self) -> et.Element:
        """Builds the basic XML structure of the comment question."""
        root = et.Element("commentQuestion")

        # Insert immediate children to the root.
        static_children: Dict[str, str] = {"isPersonalize": "false", 
                                            "QuestionDisplayType": "All", 
                                            "ID": self.internal_id, 
                                            "IdentifyID": self.identify_id}
        for child, value in static_children.items():
            child_xml = et.SubElement(root, child)
            child_xml.text = value
  
        # Build the questionTitle from instance variables.
        question_title_xml = et.SubElement(root, "questionTitle")
        
        caption_xml = et.SubElement(question_title_xml, "caption")
        caption_xml.set("lang", "en-US")
        caption_xml.text = self.text

        caption_id_xml = et.SubElement(caption_xml, "id")
        caption_id_xml.text = (f"questionnaire/commentBox/{self.internal_id}"
                               f"/questionTitle@id=en-US")

        return root

class RatingQuestion(Question):
    """Class to represent a rating question in a questionnaire."""

    def __init__(self, identify_id: str, text:str, scales: Dict[str, str], 
                 na: str = ""):
        self.identify_id: str = identify_id
        self.text: str = text
        self.scales = scales
        self.internal_id: str = str(uuid.uuid4())
        self.na: str = na
        self.xml: et.Element = self.instantiate_xml()
        
    def instantiate_xml(self) -> et.Element:
        """Builds the basic XML structure of the rating question."""
        root = et.Element("ratingQuestion")

        # Insert immediate children to the root.
        static_children: Dict[str, str] = {"isPersonalize": "false", 
                                           "QuestionDisplayType": "All", 
                                           "ID": self.internal_id, 
                                           "IdentifyID": self.identify_id, 
                                           "isSingleRow": "true"}
        for child, value in static_children.items():
            child_xml = et.SubElement(root, child)
            child_xml.text = value
  
        # Build the questionTitle from instance variables.
        question_title_xml = et.SubElement(root, "questionTitle")
        
        caption_xml = et.SubElement(question_title_xml, "caption")
        caption_xml.set("lang", "en-US")
        caption_xml.text = self.text

        caption_id_xml = et.SubElement(caption_xml, "id")
        caption_id_xml.text = (f"questionnaire/ratingQuestion/"
                               f"{self.internal_id}/questionTitle@id=en-US")

        # Build the primaryScale xml.
        primary_scale_xml = et.SubElement(root, "primaryScale")
        display_as_radio_xml = et.SubElement(primary_scale_xml, 
                                             "displayAsRadioButton")
        display_as_radio_xml.text = "true"

        i: int = 1
        for score, scale in self.scales.items():
            scale_xml = self._build_scale(score, scale, str(i))
            primary_scale_xml.append(scale_xml)
            i += 1

        # Handle the NA value (same as a scale with a blank score).
        na_scale_xml = self._build_scale("", self.na, str(i))
        primary_scale_xml.append(na_scale_xml)
        
        return root
    
    def _build_scale(self, score: str, scale: str, index: str) -> et.Element:
        """Helper function to create a scale element."""
        scale_xml = et.Element("scale")
        scale_xml.set("mandatory", "true")

        scale_caption_xml = et.SubElement(scale_xml, "caption")
        scale_caption_xml.set("lang", "en-US")

        scale_id = et.SubElement(scale_caption_xml, "id")
        scale_id.text = (f"questionnaire/ratingQuestion/{self.internal_id}"
                         f"/scale{index}@id=en-US")
        scale_id.tail = scale

        score_xml = et.SubElement(scale_xml, "score")
        score_xml.text = score

        return scale_xml


class SectionBox:
    """Class to represent a section title in a questionnaire."""

    def __init__(self, text:str):
        self.text = text
        self.internal_id: str = str(uuid.uuid4())
        self.xml: et.Element = self.instantiate_xml()

    def instantiate_xml(self) -> et.Element:
        """Builds the basic XML structure of the section title."""
        root = et.Element("sectionBox")

        # Build Identify ID XML.
        child_xml = et.SubElement(root, "ID")
        child_xml.text = self.internal_id

        # Build Question Title XML.
        question_title_xml = et.SubElement(root, "questionTitle")

        caption_xml = et.SubElement(question_title_xml, "caption")
        caption_xml.set("lang", "en-US")
        caption_xml.text = self.text

        caption_id_xml = et.SubElement(caption_xml, "id")
        caption_id_xml.text = (f"questionnaire/SectionBox/{self.internal_id}"
                               f"/questionTitle@id=en-US")
        return root

    def get_xml(self) -> et.Element:
        """Returns the xml structure of the comment question."""
        return self.xml
    
    def pretty_print(self) -> str:
        """Print the XML structure as a string with indents."""
        et.indent(self.xml)
        return et.tostring(self.xml).decode()


if __name__ == "__main__":
    test = SectionBox("Test Section Title")
    print(test.pretty_print())


import bqg
import unittest
import xml.etree.ElementTree as et
from typing import Dict

class TestCommentQuestion(unittest.TestCase):

    def setUp(self) -> None:
        self.cq = bqg.CommentQuestion("ID1","Test Question?")
        self.xml = self.cq.get_xml()

    def test_comment_returns_xml(self):

        self.assertIsInstance(self.xml, et.Element)

    def test_comment_xml_has_right_static_values(self):

        # Check all tags exist.
        self.assertEqual(self.xml.tag, "commentQuestion")

        # Check static children of commentQuestion.
        children: Dict[str, str] = {"isPersonalize":"false","QuestionDisplayType":"All"}
        for child, value in children.items():
            child_xml = self.xml.find(child)
            self.assertIsNotNone(child_xml, f"Child {child} was not found.")

            self.assertEqual(child_xml.text, value, f"Value {value} for child {child} does not exist")
        
    def test_comment_internal_id_is_uuid_format(self):
        # Check that UUID exists.
        id_xml = self.xml.find("ID")
        self.assertIsNotNone(id_xml)

        # Check that the UUID matches the regex form.
        uuid = id_xml.text
        self.assertRegex(uuid, r"^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$")
        
    def test_identify_id_forms_properly(self):
        identify_id = self.xml.find("IdentifyID").text

        self.assertEqual(identify_id,"ID1")

    def test_question_title_forms_properly(self):
        question_title = self.xml.find("questionTitle")

        self.assertIsNotNone(question_title)
        question_text = question_title.find("caption").text
        question_id = question_title.find("caption").find("id").text

        self.assertEqual(question_text,"Test Question?")

        self.assertRegex(question_id, r"^questionnaire/commentBox/[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}/questionTitle@id=en-US$")

class TestRatingQuestion(unittest.TestCase):

    def setUp(self) -> None:
        options: Dict[str, str] = {"1": "Yes", "2": "No"}
        self.cq = bqg.RatingQuestion("ID1","Test Question?", options, "")
        self.xml = self.cq.get_xml()

    def test_returns_xml(self):

        self.assertIsInstance(self.xml, et.Element)

    def test_xml_has_right_static_values(self):

        # Check all tags exist.
        self.assertEqual(self.xml.tag, "ratingQuestion")

        # Check static children of commentQuestion.
        children: Dict[str, str] = {"isPersonalize":"false","QuestionDisplayType":"All"}
        for child, value in children.items():
            child_xml = self.xml.find(child)
            self.assertIsNotNone(child_xml, f"Child {child} was not found.")

            self.assertEqual(child_xml.text, value, f"Value {value} for child {child} does not exist")
        
    def test_internal_id_is_uuid_format(self):
        # Check that UUID exists.
        id_xml = self.xml.find("ID")
        self.assertIsNotNone(id_xml)

        # Check that the UUID matches the regex form.
        uuid = id_xml.text
        self.assertRegex(uuid, r"^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$")
        
    def test_identify_id_forms_properly(self):
        identify_id = self.xml.find("IdentifyID").text

        self.assertEqual(identify_id,"ID1")

    def test_question_title_forms_properly(self):
        question_title = self.xml.find("questionTitle")

        self.assertIsNotNone(question_title)
        question_text = question_title.find("caption").text
        question_id = question_title.find("caption").find("id").text

        self.assertEqual(question_text,"Test Question?")

        self.assertRegex(question_id, r"^questionnaire/ratingQuestion/[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}/questionTitle@id=en-US$")

    def test_rating_static_values_form_properly(self):
        child_xml = self.xml.find("isSingleRow")

        self.assertIsNotNone(child_xml)
        self.assertEqual(child_xml.text, "true")

    def test_primary_scale_forms_properly(self):
        primary_scale = self.xml.find("primaryScale")
        self.assertIsNotNone(primary_scale)

        display_as_radio_button = primary_scale.find("displayAsRadioButton").text
        self.assertEqual(display_as_radio_button,"true")

        scales = primary_scale.findall("scale")
        scale_scores = [x.find("score").text for x in scales]
        self.assertEqual(scale_scores, ["1","2",""])

class TestSectionBox(unittest.TestCase):

    def setUp(self) -> None:
        self.sb = bqg.SectionBox("Test Question?")
        self.xml = self.sb.get_xml()

    def test_returns_xml(self):
        self.assertIsInstance(self.xml, et.Element)

    def test_internal_id_is_uuid_format(self):
        # Check that UUID exists.
        id_xml = self.xml.find("ID")
        self.assertIsNotNone(id_xml)

        # Check that the UUID matches the regex form.
        uuid = id_xml.text
        self.assertRegex(uuid, r"^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$")
        
if __name__ == "__main__":
    unittest.main()